#include <os/File.h>
#include <os/Process.h>
#include <signal.h>
#include <sys/wait.h>
#include <ui/App.h>
#include <ui/Button.h>
#include <ui/Container.h>
#include <ui/Image.h>
#include <ui/Layout.h>

void sigchld_handler(int)
{
    wait(nullptr);
}

Result<int> luna_main(int argc, char** argv)
{
    ui::App app;
    TRY(app.init(argc, argv));

    signal(SIGCHLD, sigchld_handler);

    ui::Rect screen = app.screen_rect();

    ui::Rect bar = ui::Rect { ui::Point { 0, screen.height - 50 }, screen.width, 50 };

    auto window = TRY(ui::Window::create(bar, false));
    app.set_main_window(window);
    window->set_background(ui::GRAY);

    ui::HorizontalLayout layout(ui::AdjustHeight::Yes, ui::AdjustWidth::No);
    window->set_main_widget(layout);

    ui::Button term_button({ 0, 0, 50, 50 });
    layout.add_widget(term_button);

    ui::Container term_container({ 0, 0, 50, 50 }, ui::VerticalAlignment::Center, ui::HorizontalAlignment::Center);
    term_button.set_widget(term_container);
    term_button.set_action([] {
        StringView args[] = { "/usr/bin/terminal" };
        os::Process::spawn("/usr/bin/terminal", Slice<StringView> { args, 1 }, false);
    });

    auto term_image = TRY(ui::ImageWidget::load("/usr/share/icons/32x32/app-terminal.tga"));
    term_container.set_widget(*term_image);

    ui::Button about_button({ 0, 0, 50, 50 });
    layout.add_widget(about_button);

    ui::Container about_container({ 0, 0, 50, 50 }, ui::VerticalAlignment::Center, ui::HorizontalAlignment::Center);
    about_button.set_widget(about_container);
    about_button.set_action([] {
        StringView args[] = { "/usr/bin/about" };
        os::Process::spawn("/usr/bin/about", Slice<StringView> { args, 1 }, false);
    });

    auto about_image = TRY(ui::ImageWidget::load("/usr/share/icons/32x32/app-about.tga"));
    about_container.set_widget(*about_image);

    window->draw();

    return app.run();
}
