#include "fs/devices/SlavePTY.h"
#include "Pledge.h"
#include "fs/devices/MasterPTY.h"
#include "memory/MemoryManager.h"
#include "thread/Scheduler.h"

Result<usize> SlavePTY::read(u8* buf, usize, usize length) const
{
    if (!m_master) return err(EIO);

    TRY(m_master->handle_background_process_group(false, SIGTTIN));

    length = m_buffer.dequeue_data(buf, length);

    return length;
}

Result<usize> SlavePTY::write(const u8* buf, usize, usize length)
{
    if (!m_master) return err(EIO);

    if (m_master->m_settings.c_lflag & TOSTOP) TRY(m_master->handle_background_process_group(true, SIGTTOU));

    TRY(m_master->m_buffer.append_data(buf, length));

    return length;
}

Result<u64> SlavePTY::ioctl(int request, void* arg)
{
    auto* current = Scheduler::current();
    TRY(check_pledge(current, Promise::p_tty));

    if (!m_master) return err(EIO);

    switch (request)
    {
    case TCGETS: {
        return MemoryManager::copy_to_user_typed((struct termios*)arg, &m_master->m_settings) ? 0 : err(EFAULT);
    }
    case TCSETS: {
        if (!MemoryManager::copy_from_user_typed((const struct termios*)arg, &m_master->m_settings)) return err(EFAULT);

        return 0;
    }
    case TIOCSPGRP: {
        pid_t pgid;
        if (!MemoryManager::copy_from_user_typed((const pid_t*)arg, &pgid)) return err(EFAULT);

        bool pgid_exists = false;
        Scheduler::for_each_in_process_group(pgid, [&pgid_exists](Thread*) {
            pgid_exists = true;
            return false;
        });
        if (!pgid_exists) return err(EPERM);

        m_master->m_foreground_process_group = pgid;
        return 0;
    }
    case TIOCGPGRP: {
        pid_t pgid = m_master->m_foreground_process_group.value_or((pid_t)next_thread_id());
        if (!MemoryManager::copy_to_user_typed((pid_t*)arg, &pgid)) return err(EFAULT);
        return 0;
    }
    case TIOCGWINSZ: {
        if (!MemoryManager::copy_to_user_typed((struct winsize*)arg, &m_master->m_window)) return err(EFAULT);
        return 0;
    }
    default: return err(EINVAL);
    }
}
