#pragma once
#include "fs/VFS.h"
#include "fs/devices/DeviceRegistry.h"
#include "fs/devpts/FileSystem.h"

namespace DevPTS
{
    class RootInode : public VFS::Inode
    {
      public:
        RootInode() = default;

        void set_fs(FileSystem& fs, Badge<FileSystem>)
        {
            m_fs = &fs;
        }

        void set_inode_number()
        {
            m_metadata.inum = 2;
        }

        void set_self(SharedPtr<VFS::Inode> self, Badge<FileSystem>)
        {
            m_self = self;
        }

        Result<SharedPtr<VFS::Inode>> find(const char* name) const override;
        Option<VFS::DirectoryEntry> get(usize index) const override;

        Result<usize> read(u8*, usize, usize) const override
        {
            return err(EISDIR);
        }

        Result<usize> write(const u8*, usize, usize) override
        {
            return err(EISDIR);
        }

        Result<void> truncate(usize) override
        {
            return err(EISDIR);
        }

        bool will_block_if_read() const override
        {
            return false;
        }

        VFS::FileSystem* fs() const override
        {
            return m_fs;
        }

        VFS::InodeType type() const override
        {
            return VFS::InodeType::Directory;
        }

        void did_link() override
        {
        }

        void did_unlink() override
        {
            m_self = {};
            m_entries.clear();
        }

        usize entries() const override
        {
            return m_entries.size();
        }

        Result<void> remove_entry(const char* name) override;

        Result<SharedPtr<VFS::Inode>> create_file(const char* name, mode_t mode) override;
        Result<SharedPtr<VFS::Inode>> create_subdirectory(const char* name, mode_t mode) override;

        Result<void> add_entry(SharedPtr<VFS::Inode> inode, const char* name);
        Result<void> replace_entry(SharedPtr<VFS::Inode> inode, const char* name);

        virtual ~RootInode() = default;

      private:
        VFS::FileSystem* m_fs;
        SharedPtr<VFS::Inode> m_self;
        Vector<VFS::DirectoryEntry> m_entries;

        friend class FileSystem;
    };
}
